options(stringsAsFactors=FALSE)
library(biomaRt)
library(tidyverse)
library(stringr)
library(cowplot)

source(file="settings.R")

DATA_DIR = '../SourceData/'

# Helper function: To lowercase and remove special characters
trim = function(x) {
  return(gsub("[._ -]","",tolower(x)))
}

getListItem = function(l, item) {
  if(item %in% names(l))
    return(l[[item]])
  else
    return(FALSE)
}

listItemExists = function(l, key) {
  return(key %in% names(l))
}

listItemEquals = function(l, key, val) {
  if(listItemExists(l, key)) 
    if(l[[key]]==val)
      return(T)
  return(F)
}

listItemContains = function(l, key, val) {
  if(listItemExists(l, key)) 
    if(grepl(val, l[[key]]))
      return(T)
  return(F)
}


##########################################################
## READ CELLOSAURUS
## Motivation:
## - only include one cell per patient
## - only include "most parental" cell line from a hierachy
## - remove problematic cell lines
## 
## Note: Run time ~15 minutes
###########################################################
print(paste(Sys.time(), ": Downloading Cellosaurus", sep=""))
#con = file(paste(DATA_DIR,"cellosaurus.txt",sep=""), "r") # http://web.expasy.org/cellosaurus/
con = url("ftp://ftp.expasy.org/databases/cellosaurus/cellosaurus.txt")
cell_line_ids = c()
cell_lines_samepatient = list()
cell_line_ac = list()
cell_line_synonyms = list()
cell_line_canonical_untrimmed = list()
cell_line_parents = list()
cell_line_daughters = list()
cell_line_species = list()
cell_lines_problematic = c()
tmp_samepatient = c()
tmp_parents = c()
tmp_synonyms = c()
species = ""
skip = TRUE
id = ""
ac = ""
print(paste(Sys.time(), ": Processing Cellosaurus", sep=""))
lines = readLines(con)
for(line in lines) {
  line = str_trim(line)
  if(str_detect(line, "^ID")) {
    id = str_match(line, "^ID\\s+(.+)$")[2]
    id_untrimmed = id
    id = trim(id)
  } else if(str_detect(line, "^AC")) {
    ac = str_match(line, "^AC\\s+(.+)$")[2]
  }
  else if(str_detect(line, "Problematic cell line")) {
    cell_lines_problematic = c(cell_lines_problematic,id)
  } else if (str_detect(line, "^OI.+!\\s+(.+)$")) {
    tmp_samepatient = c(tmp_samepatient, str_trim(str_match(line, "^OI.+!\\s+(.+)$")[2] ))
  } else if (str_detect(line, "^SY\\s+(.+)$")) {
      str_match(line, "^SY\\s+(.+)$")[2] %>% 
      str_trim %>% 
      strsplit(";") %>% 
      unlist %>% 
      str_trim -> tmp_synonyms
  } else if (str_detect(line, "^HI.+!\\s+(.+)$")) {
    parent = trim(str_match(line, "^HI.+!\\s+(.+)$")[2])
    if(parent %in% names(cell_line_daughters)) {
      cell_line_daughters[[parent]] = c(cell_line_daughters[[parent]], id)
    } else {
      cell_line_daughters[[parent]] = c(id)
    }
    tmp_parents = c(tmp_parents, parent )
  } else if (str_detect(line, "^OX.+!\\s+(.+)$")) {
    species =  str_trim(str_match(line, "^OX.+!\\s+(.+)$")[2])
  } else if(str_detect(line, "^//$")) {
    if(!skip) {
      if(id=="") { print("Error: missing cell line id") }
      #if(id %in% cell_line_species) { print(paste("Duplicate cell line: ", id, sep="")) }
      cell_lines_samepatient[[id]] = trim(tmp_samepatient)
      cell_line_parents[[id]] = trim(tmp_parents)
      cell_line_species[[id]] = species
      cell_line_synonyms[[id]] = trim(tmp_synonyms)
      cell_line_canonical_untrimmed[[id]]=id_untrimmed
      cell_line_ac[[id]] = ac
      cell_line_ids = c(cell_line_ids, id)
    }
    skip = FALSE
    tmp_samepatient = c()
    tmp_parents = c()
    tmp_synonyms = c()
    id = ""
    species = ""
  }
}
close(con)

# some additional cell line parent information
cell_line_parents[["a375r18"]]=c("a375")
cell_line_parents[["a375r37"]]=c("a375")
cell_line_parents[["a375matchedparental"]]=c("a375")
cell_line_parents[["a375r15"]]=c("a375")
cell_line_parents[["a375r16"]]=c("a375")
cell_line_parents[["a375r35"]]=c("a375")
cell_line_parents[["a375r36"]]=c("a375")
cell_line_parents[["a375r38"]]=c("a375")
cell_line_parents[["a375r51"]]=c("a375")
cell_line_parents[["a375r65"]]=c("a375")
cell_line_parents[["a375dr4"]]=c("a375")
cell_line_parents[["a375dr7"]]=c("a375")
cell_line_parents[["a375dr12"]]=c("a375")
cell_line_parents[["ht144dr5"]]=c("ht144")
cell_line_parents[["ht144dr6"]]=c("ht144")

# additional problematic cell lines
#cell_lines_problematic = c()


##############################
###### HELPER FUNCTIONS ######
##############################

# construct cell line families from parental information
#  - note: use canonical names
create_families = function(CLs) {
  
  track_parents = function(CL, lineage) {
    if(CL %in% names(cell_line_parents)) {
      if(length(cell_line_parents[[CL]])>0) {
        #print(paste("Parent: ", cell_line_parents[[CL]][1], sep=""))
        return(track_parents(cell_line_parents[[CL]][1], c(cell_line_parents[[CL]][1], lineage)))
      }
    }  
    return(lineage)
  } 
  
  track_daughters = function(CL, pool=c()) {
    new_pool = pool
    if(CL %in% names(cell_line_daughters)) {
      daughters = cell_line_daughters[[CL]]
      if(length(daughters)>0) {
        for(daughter in daughters) {
          if(!(daughter %in% new_pool)) {
            new_pool = c(new_pool, daughter)
            new_pool = unique(c(new_pool,  track_daughters(daughter, new_pool)))
          }
        }
      }
    }
    return(new_pool)
  } 
  
  lineages = list()
  for(CL in CLs) {
    lineages[[CL]] = c(track_parents(CL, lineage=c(CL)), track_daughters(CL))
  }
  
  families = list()
  for(CL1 in CLs) {
    families[[CL1]] = c()
    for(CL2 in CLs) {
      if(length(intersect(lineages[[CL1]],lineages[[CL2]]))>0) {
        families[[CL1]] = c(families[[CL1]], CL2)
      }
    }
  }
  return(families)
}

# Rename cell line name to canonical name
rename_canonical <- function(CLs) {
  ret = c()
  for(CL in CLs) {
    CL = trim(CL)
    found = FALSE
    if(CL %in% names(cell_line_synonyms)) {
      ret = c(ret,CL)
      found = TRUE
    } else
      for(i in seq(1,length(cell_line_synonyms))) {
        if(CL %in% cell_line_synonyms[[i]]) {
          ret = c(ret,names(cell_line_synonyms)[i])
          found = TRUE
          break
        }
      }
    if(!found) {
      ret = c(ret,CL)
    }
  }
  return(ret)
}

CL_pub_label <- function(CLs) {
  ret = c()
  for(CL in CLs) {
    CL = trim(CL)
    if(CL %in% names(CL_pub_label_list)) {
      ret=c(ret,CL_pub_label_list[[CL]])
    }
    else if(CL %in% names(cell_line_canonical_untrimmed)) {
      ret = c(ret,cell_line_canonical_untrimmed[[CL]])
    } else
          ret = c(ret,CL)
      }
  ret = gsub(" ", "", ret, fixed = TRUE)
  return(ret)
}


# Remove daugher cell lines from data frame
remove_daughters <- function(df) {
  remove_rows = c()
  for(i in seq(1,nrow(df))) {
    cl = df$Cell.Line[i]
    cl = trim(cl)
    if(cl %in% names(cell_line_parents)) {
      if(any(cell_line_parents[[cl]] %in% trim(df$Cell.Line))) {
        remove_rows = c(remove_rows, i)
      }
    }
  }
  if(length(remove_rows)>0) {
    df = df[-remove_rows,]
  }
  return(df)
}

# Remove non-human cell lines
remove_nonhuman <- function(df) {
  remove_rows = c()
  for(i in seq(1,nrow(df))) {
    cl = trim(df$Cell.Line[i])
    if(cl %in% names(cell_line_species)) {
      if(cell_line_species[[cl]] != "Homo sapiens") {
        remove_rows = c(remove_rows, i)
      }
    }
  }
  if(length(remove_rows)>0) {
    df = df[-remove_rows,]
  }
  return(df)
}


##############################################################
## READ VIAIBLITY 
## - filter out non-human cell lines
## - filter out problematic cell lines
## - filter out cell lines whose parents are also in the panel
## - use canonical cell line id
###############################################################
# GENENTECH

# melanoma
print(paste(Sys.time(), ": Reading GT viability", sep=""))
sensitivity_melanoma = read.csv(file=paste(DATA_DIR,"Melanoma Sensitivity Data.csv",sep=""), check.names = F)
colnames(sensitivity_melanoma)[1] = "Cell.Line"

print(paste(Sys.time(), ": Processing GT viability", sep=""))
sensitivity_melanoma %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  #remove_nonhuman %>% 
  #filter(!(Cell.Line %in% cell_lines_problematic)) %>% 
  group_by(Cell.Line) %>% 
  summarise_all(funs(mean(., na.rm = TRUE))) %>% 
  arrange(Cell.Line) -> sensitivity_melanoma

sensitivity_melanoma = data.frame(sensitivity_melanoma, check.names=F)
rownames(sensitivity_melanoma) = sensitivity_melanoma$Cell.Line

# endometrial
sensitivity_endometrial = read.csv(file=paste(DATA_DIR,"Endometrial_Drug_Data_drugsel.csv",sep=""), check.names = F)
sensitivity_endometrial %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  group_by(Cell.Line, Drug) %>% 
  summarise(Viability=mean(Viability, na.rm=T)) %>% 
  spread(Drug, Viability) %>% 
  arrange(Cell.Line) -> sensitivity_endometrial

sensitivity_endometrial = data.frame(sensitivity_endometrial, check.names=F)
rownames(sensitivity_endometrial) = sensitivity_endometrial$Cell.Line


# CCLE
print(paste(Sys.time(), ": Downloading CCLE viability", sep=""))
#sen_ccle = read.csv(file=paste(DATA_DIR,"CCLE_NP24.2009_Drug_data_2015.02.24.csv", sep=""), check.names = F)
sen_ccle = read.csv(file=url("https://data.broadinstitute.org/ccle_legacy_data/pharmacological_profiling/CCLE_NP24.2009_Drug_data_2015.02.24.csv"), check.names = F)
sen_ccle = sen_ccle[,c(1,3,4,13)]
colnames(sen_ccle)[1] = "Cell.Line"
sen_ccle = sen_ccle %>% extract(Cell.Line, into=c("Cell.Line","tissue"), regex='(^[:alnum:]+)_(.*)' )

print(paste(Sys.time(), ": Processing CCLE viability", sep=""))
sen_ccle %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( tissue = trim(tissue)) %>% 
  mutate( Compound = trim(Compound)) %>% 
  mutate( Target = trim(Target)) -> sen_ccle

drugTargets_ccle = sen_ccle %>% dplyr::select(Compound,Target) %>% distinct
sen_ccle$Target = NULL
sen_ccle %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  filter(!is.na(Cell.Line)) %>% 
  group_by(Cell.Line, tissue, Compound) %>% 
  summarise(via=mean(ActArea)) %>% 
  spread(Compound,via) %>% 
  arrange(Cell.Line) -> sen_ccle
sen_ccle = data.frame(sen_ccle, check.names = F)
rownames(sen_ccle) = sen_ccle$Cell.Line

drugTargets_ccle_curated = read.csv(file=paste(DATA_DIR,"drug_targets_ccle_curated.csv",sep=""))


# CTRP
print(paste(Sys.time(), ": Downloading CTRP viability", sep=""))
tmpfile = tempfile()
download.file("ftp://anonymous:guest@caftpd.nci.nih.gov/pub/OCG-DCC/CTD2/Broad/CTRPv2.2_2015_pub_CancerDisc_5_1210/CTRPv2.2_2015_pub_CancerDisc_5_1210.zip", tmpfile, method="wget")

#ctrp_drug_idx = read.csv(file=paste(DATA_DIR,"v22.meta.per_compound.txt",sep=""), check.names = F, sep="\t")
ctrp_drug_idx = read.csv(unzip(tmpfile,files="v22.meta.per_compound.txt"), check.names = F, sep="\t")
ctrp_drug_idx = ctrp_drug_idx[,c("index_cpd", "cpd_name", "target_or_activity_of_compound")]
colnames(ctrp_drug_idx)[3] = "Target"
#ctrp_cl_idx = read.csv(file=paste(DATA_DIR,"v22.meta.per_cell_line.txt",sep=""), check.names = F, sep="\t")
ctrp_cl_idx = read.csv(unzip(tmpfile,"v22.meta.per_cell_line.txt"), check.names = F, sep="\t")
ctrp_cl_idx = ctrp_cl_idx[,c("index_ccl", "ccl_name", "ccle_primary_site")]
colnames(ctrp_cl_idx)[3]="tissue"
#sen_ctrp =read.csv(file=paste(DATA_DIR,"v22.data.auc_sensitivities.txt",sep=""), check.names = F, sep="\t")
sen_ctrp =read.csv(unzip(tmpfile,"v22.data.auc_sensitivities.txt"), check.names = F, sep="\t")
colnames(sen_ctrp)[3] = "AUC"
unlink(tmpfile)

print(paste(Sys.time(), ": Processing CTRP viability", sep=""))
sen_ctrp = sen_ctrp %>% 
  left_join(ctrp_drug_idx) %>% 
  left_join(ctrp_cl_idx) %>% 
  dplyr::select(-index_cpd, -index_ccl) %>% 
  mutate( cpd_name = trim(cpd_name)) %>% 
  group_by(cpd_name, ccl_name, Target, tissue) %>% 
  summarise(AUC = mean(AUC, na.rm=T)) %>% 
  ungroup 

drugTargets_ctrp = sen_ctrp %>% dplyr::select(cpd_name,Target) %>% distinct
drugTargets_ctrp_curated = read.csv(file=paste(DATA_DIR,"drug_targets_ctrp_curated.csv",sep=""))

sen_ctrp = sen_ctrp %>% 
  dplyr::select(-Target) %>% 
  spread(cpd_name, AUC) %>% 
  mutate( tissue = trim(tissue)) %>% 
  rename(Cell.Line = ccl_name) %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  arrange(Cell.Line)
  
sen_ctrp = data.frame(sen_ctrp, check.names = F)
rownames(sen_ctrp) = sen_ctrp$Cell.Line

##############################################################
## READ RNASEQ DATA. 
## - average duplicate cell lines
## - filter out non-human cell lines
## - filter out problematic cell lines
## - filter out cell lines whose parents are also in the panel
## - use canonical cell line id
###############################################################
print(paste(Sys.time(), ": Reading GT RNAseq data", sep=""))
rnaseq = read.csv(file=paste(DATA_DIR,"Melanoma_Endometrial_RNASeq.csv",sep=""))
rnaseq$ColumnName = NULL
rnaseq_melanoma = rnaseq[rnaseq$Cancer.Type=="Melanoma",]
rnaseq_melanoma$Cancer.Type=NULL
print(paste(Sys.time(), ": Processing GT melanoma RNAseq data", sep=""))
rnaseq_melanoma %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  #remove_nonhuman %>% 
  #filter(!(Cell.Line %in% cell_lines_problematic)) %>% 
  group_by(Cell.Line) %>% 
  summarise_all(funs(mean(., na.rm = TRUE))) %>% 
  arrange(Cell.Line) -> rnaseq_melanoma
rnaseq_melanoma = data.frame(rnaseq_melanoma, check.names=F)
rownames(rnaseq_melanoma) = rnaseq_melanoma[,1]

print(paste(Sys.time(), ": Processing GT endometrial RNAseq data", sep=""))
rnaseq_endometrial = rnaseq[rnaseq$Cancer.Type=="Endometrial",]
rnaseq_endometrial$Cancer.Type=NULL
rnaseq_endometrial = as_tibble(rnaseq_endometrial)
rnaseq_endometrial %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  #remove_nonhuman %>% 
  #filter(!(Cell.Line %in% cell_lines_problematic)) %>% 
  group_by(Cell.Line) %>% 
  summarise_all(funs(mean(., na.rm = TRUE))) %>% 
  arrange(Cell.Line) -> rnaseq_endometrial
rnaseq_endometrial = data.frame(rnaseq_endometrial, check.names=F)
rownames(rnaseq_endometrial) = rnaseq_endometrial[,1]

# CCLE
print(paste(Sys.time(), ": Downloading CCLE RNAseq data", sep=""))
#rnaseq_ccle = read_tsv(file=paste(DATA_DIR,"CCLE_DepMap_18Q1_RNAseq_RPKM_20180214.gct",sep=""), skip=2)
rnaseq_ccle = read_tsv(file=url("https://data.broadinstitute.org/ccle/CCLE_DepMap_18Q1_RNAseq_RPKM_20180214.gct"), skip=2)
#  rnaseq_ccle_head.gct

print(paste(Sys.time(), ": Processing CCLE RNAseq data", sep=""))
rnaseq_ccle = rnaseq_ccle %>% 
  dplyr::select(-Description) %>% 
  gather("Cell.Line", "expr", -Name) %>% 
  mutate(expr = log2(1+expr)) %>% 
  spread(Name, expr) %>% 
  mutate(tissue = gsub('^.*?_', '', Cell.Line)) %>% 
  mutate(tissue = trim(tissue)) %>% 
  mutate( Cell.Line = gsub('_.*', '',Cell.Line)) %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  group_by(Cell.Line, tissue) %>% 
  summarise_all(funs(mean(., na.rm = TRUE))) %>% 
  ungroup %>% 
  distinct(Cell.Line, .keep_all = TRUE) %>% 
  dplyr::select(Cell.Line, tissue, everything()) %>% 
  arrange(Cell.Line)

rnaseq_ccle = data.frame(rnaseq_ccle, check.names=F)
rownames(rnaseq_ccle) = rnaseq_ccle$Cell.Line


##########################################################
## READ COSMIQ CANCER GENE CENSUS
## READ RPPA_CCLE GENE IDs
## SPEED1
## 
###########################################################
print(paste(Sys.time(), ": Reading COSMIC", sep=""))

# Download this file from COSMIC Cancer Gene Census. Need to register.
cosmic = read.csv(file=paste(DATA_DIR,"cancer_gene_census.csv",sep=""))
cosmic_genesym = cosmic$Gene.Symbol
cosmic_genesym = cosmic_genesym[!is.na(cosmic_genesym)]
cosmic = cosmic$Synonyms
cosmic = str_match(cosmic, "(ENSG\\d+)")
cosmic = cosmic[,1]
cosmic = cosmic[!is.na(cosmic)]

gt_rppa_skin_genesym = read.csv(file=paste(DATA_DIR,"prot2sym_gt_skin.csv",sep=""))
gt_rppa_skin_genesym = gt_rppa_skin_genesym$sym
gt_rppa_skin_genesym = unique(gt_rppa_skin_genesym)

rppa_ccle_ensg = read.csv(file=paste(DATA_DIR,"prot2ensg_ccle.csv",sep=""))
rppa_ccle_ensg = rppa_ccle_ensg$ENSG
rppa_ccle_ensg = unique(rppa_ccle_ensg)

speed1 = read.csv(file=paste(DATA_DIR,"speed1_default.txt",sep=""), sep="\t")
colnames(speed1) = c("pathway", "entrez_id", "gene_symbol")
speed1$pathway =  gsub("[._ -]","", tolower(speed1$pathway))

rnaseq_ccle_ensembl = str_split(colnames(rnaseq_ccle),"\\.", simplify = T)[,1][-c(1,2)]
mart <- useDataset("hsapiens_gene_ensembl", useMart("ensembl"))
rnaseq_ccle_entrez <- getBM(filters="ensembl_gene_id", 
                            attributes=c("ensembl_gene_id", "entrezgene"),
                            values=rnaseq_ccle_ensembl,
                            mart=mart)
colnames(rnaseq_ccle_entrez) = c("ensembl_gene_id", "entrez_id")
speed1_ensembl = speed1[speed1$entrez_id %in% unique(rnaseq_ccle_entrez$entrez_id),]
speed1_ensembl = inner_join(speed1_ensembl, rnaseq_ccle_entrez)

##############################################################
## READ RPPA DATA 
## - average duplicate cell lines
## - filter out non-human cell lines
## - filter out problematic cell lines
## - filter out cell lines whose parents are also in the panel
## - use canonical cell line id
## 
## Note: There are two variants of RPPA data labelled Cobi/Vem
###############################################################
print(paste(Sys.time(), ": Reading GT proteomics data", sep=""))
rppa_gt_skin = read.csv(file=paste(DATA_DIR,"GNE208 RPPA Data.csv",sep=""))

rppa_gt_endo = read.csv(file=paste(DATA_DIR,"Endometrial_RPPA_Data.csv",sep=""))
colnames(rppa_gt_endo)[1] = "Cell.Line"

rppaPipeShort = function(df) {
  return(df %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>%
  arrange(Cell.Line))
}

rppaPipe = function(df) {
  df %>%   
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  #remove_nonhuman %>% 
  #filter(!(Cell.Line %in% cell_lines_problematic)) %>% 
  group_by(Cell.Line, Tx) %>% 
  summarise_all(funs(mean(., na.rm = TRUE))) %>% 
  arrange(Cell.Line) -> df_ret
  df_ret = data.frame(df_ret, check.names=F)
  return(df_ret)
}

print(paste(Sys.time(), ": Processing GT proteomics data", sep=""))
rppa_gt_skin %>% rppaPipe -> rppa_gt_skin

print(paste(Sys.time(), ": Downloading CCLE proteomics data", sep=""))
#rppa_ccle = read.csv(file=paste(DATA_DIR,"CCLE_RPPA_20180123.csv",sep=""), sep=",")
rppa_ccle = read_delim(file=url("https://data.broadinstitute.org/ccle/CCLE_RPPA_20180123.csv"),delim=",")
colnames(rppa_ccle)[1] = "Cell.Line"

# separate cell line name and origin
rppa_ccle = rppa_ccle %>% extract(Cell.Line, into=c("Cell.Line","tissue"), regex='(^[:alnum:]+)_(.*)' )

# double annotation, delete row
rppa_ccle = rppa_ccle[-which(rppa_ccle$Cell.Line=="TT" & rppa_ccle$tissue=="THYROID"),]

# convert to numeric
rppa_ccle = cbind(rppa_ccle[,1:2], data.frame(apply(rppa_ccle[, -c(1,2)], 2, function(x) as.numeric(as.character(x))), check.names=F))

# map: number of cell lines per tissue
#data.frame(rppa_ccle %>% group_by(origin) %>% summarise(nCLs = n()) %>% arrange(-nCLs))

print(paste(Sys.time(), ": Processing CCLE proteomics data", sep=""))
rppa_ccle %>% rppaPipeShort %>% mutate( tissue = trim(tissue)) -> rppa_ccle
rppa_gt_endo$Tx = "DMSO"
rppa_gt_endo %>% rppaPipe -> rppa_gt_endo

rownames(rppa_ccle) = rppa_ccle$Cell.Line

# match melanoma to endometrial data
rppa_gt_skin_endosel = rppa_gt_skin
ids = match(gsub("[._ -]","", tolower(colnames(rppa_gt_skin))),
            gsub("[._ -]","", tolower(colnames(rppa_gt_endo))))
rppa_gt_skin_endosel = rppa_gt_skin_endosel[,!is.na(ids)]


##############################################################
## READ MELANOMA MUTATION DATA. 
## - filter out non-human cell lines
##
## Note: three different definitions of mutations is used
#        - point mutation at a specific location of a specific
#          base into a specific base
#        - point mutation at a specific location regardless of 
#          new base identity
#        - point putation at any location within a gene
###############################################################
print(paste(Sys.time(), ": Reading GT genomics data", sep=""))
mutation_specPt = read.csv(file=paste(DATA_DIR,"melanoma_vcf_matrix_all.csv",sep=""))
mutation_unspecPt = read.csv(file=paste(DATA_DIR,"melanoma_vcf_matrixAnyPtMut_all.csv",sep=""))
mutation_gene = read.csv(file=paste(DATA_DIR,"melanoma_vcf_matrixGeneMut_all.csv",sep=""))

mutationPipe = function(df) {
  df %>% 
  mutate( Cell.Line = trim(Cell.Line)) %>% 
  mutate( Cell.Line = rename_canonical(Cell.Line)) %>% 
  #remove_nonhuman %>% 
  #filter(!(Cell.Line %in% cell_lines_problematic)) %>% 
  arrange(Cell.Line) -> df_ret
  df_ret = data.frame(df_ret, check.names=F)
  rownames(df_ret) = df_ret$Cell.Line
  return(df_ret)
}

print(paste(Sys.time(), ": Processing GT genomics data", sep=""))
mutation_specPt %>% mutationPipe -> mutation_specPt
mutation_unspecPt %>% mutationPipe -> mutation_unspecPt
mutation_gene %>% mutationPipe -> mutation_gene

print(paste(Sys.time(), ": Downloading CCLE genomics data", sep=""))
#mutation_ccle = t(read.csv(file=paste(DATA_DIR,"CCLE_mutations.txt",sep=""), sep="\t", header=F))
mutation_ccle_con = gzcon(url("http://amp.pharm.mssm.edu/static/hdfs/harmonizome/data/cclemut/gene_attribute_matrix.txt.gz"))
mutation_ccle_raw = textConnection(readLines(mutation_ccle_con))
close(mutation_ccle_con)
mutation_ccle = t(read.csv(mutation_ccle_raw, sep="\t", header=F))

mutation_ccle = mutation_ccle[-2,]
mutation_ccle = mutation_ccle[,-3]
mutation_ccle[1,1] = "Cell.Line"
mutation_ccle[1,2] = "tissue"
mutation_ccle[1,-c(1,2)] = paste(mutation_ccle[1,-c(1,2)], mutation_ccle[2,-c(1,2)], sep='_')
mutation_ccle = mutation_ccle[-2,]
colnames(mutation_ccle) = mutation_ccle[1,]
mutation_ccle = mutation_ccle[-1,]
mutation_ccle = cbind(mutation_ccle[,1:2], data.frame(apply(mutation_ccle[, -c(1,2)], 2, function(x) as.numeric(as.character(x))), check.names=F))

print(paste(Sys.time(), ": Processing CCLE genomics data", sep=""))
mutation_ccle %>% mutationPipe %>% mutate( tissue = trim(tissue)) -> mutation_ccle
rownames(mutation_ccle) = mutation_ccle$Cell.Line

##############################################################
## REMOVE DUPLICATE CELL LINES
## Some cell lines are parents/siblings/daughters, and some cell 
## lines originate from the same patient. We want to keep only
## on of them.
## - Choose cell lines such that overlap is maximal with priolist1
# (viability data). If not unique, choose cell line which is also
#    in priolist2.
# Cell line names assued to be canonical
###############################################################

remove_duplicates <- function(df, priolist1 = c(), priolist2=c()) {
  
  CLs = unique(df$Cell.Line)
  CLs_duplicates = create_families(CLs) # cell line relatives, takes a while
  CLs_duplicates2 = CLs_duplicates
  
  # add cell lines that originaate from the same patient to duplicates
  for(CL in CLs) {
    if(CL %in% names(cell_lines_samepatient)) {
      CLs_duplicates[[CL]] = unique(c(CLs_duplicates[[CL]], intersect(CLs, cell_lines_samepatient[[CL]] ) ) )
    }
  }
  CLs_keep = c()
  CLs_remove = c()
  for(CL in CLs) {
    if(CL %in% CLs_remove) next
    if(length(CLs_duplicates[[CL]]) == 1) { 
      CLs_keep = c(CLs_keep, CL) 
    } else {
      prio_CL = CL
      prio_CLs = intersect( CLs_duplicates[[CL]], priolist1 )
      if(length(prio_CLs)>0) {
        prio_CL = prio_CLs[1]
        prio_CLs2 = intersect( prio_CLs, priolist2 )
        if(length(prio_CLs2)>0) {
          prio_CL = prio_CLs2[1]
        }
      }
      CLs_keep = c(CLs_keep, prio_CL) 
      CLs_remove = c(CLs_remove, setdiff(CLs_duplicates[[CL]], prio_CL))
    }
  }
  if(length(CLs_keep)>0) {
    df = df[df$Cell.Line %in% CLs_keep, ]
  }
  return(df)
}

priolist1 = sensitivity_melanoma$Cell.Line
priolist2 = intersect(rnaseq_melanoma$Cell.Line, rppa_gt_skin$Cell.Line)


##################################################################
## SEPARATE MEAN VIABILITY / IC50 / MAX INHIBITION SENSITIVTY DATA
##################################################################

sensitivity_melanoma %>% 
  dplyr::select(starts_with("Cell.Line"), starts_with("Mean Viability")) %>% 
  rename_at(vars(starts_with('Mean Viability ')), funs(sub('Mean Viability ', '', .))) -> sensitivity_melanoma_viability
  
sensitivity_melanoma %>% 
  dplyr::select(starts_with("Cell.Line"), starts_with("IC50")) %>% 
  rename_at(vars(starts_with('IC50')), funs(sub('IC50 ', '', .))) -> sensitivity_melanoma_ic50


sensitivity_melanoma %>% 
  dplyr::select(starts_with("Cell.Line"), starts_with("Max.Inhibition")) %>% 
  rename_at(vars(starts_with('Max Inhibition ')), funs(sub('Max Inhibition ', '', .))) -> sensitivity_melanoma_maxinh


##########################################################
## RETURN RPPA/RNASEQ/VIABILITY/MUTATION DATA
## - Input: Should cell lines be matched to another dataset?
## - Input: Should signals be matched to another dataset?
###########################################################

getData = function(ds, remove_dups=NULL, match_CLs=NULL, remove_tissue_column=T) {
  df=NULL
  # match_CLs should be a list of lists
  if(length(match_CLs)>0 & typeof(match_CLs[[1]])!="list")
    match_CLs = list(match_CLs)
  
  if(ds$datatype=="rppa") {
    df = getRPPA(ds)
  } else if(ds$datatype=="rnaseq") {
    df = getRNAseq(ds)
  } else if(ds$datatype=="mut") {
    df = getMutations(ds)
  } else if(ds$datatype=="sen") {
    df = getSensitivity(ds)
  } else if(ds$datatype=="tissue") {
    df = getTissue(ds)
  }
  
  df = data.frame(df, check.names=F)
  
  if(!is.null(remove_dups))
    if(remove_dups)
      df = remove_duplicates(df)
  
  if(length(match_CLs)>0) {
    sharedCLs = reduce(lapply(match_CLs, function(dx) rownames(getData(dx, match_CLs=NULL, remove_dups=remove_dups))),intersect)
    df = df[df$Cell.Line %in% sharedCLs,]
  }
    
  if(listItemExists(ds, 'tissue') & ('tissue' %in% colnames(df)) ) {
    if(ds$tissue != "all")  
      df = df[df$tissue %in% ds$tissue,]
  }
    
  if(remove_tissue_column & ('tissue' %in% colnames(df)) & (ds$datatype!="tissue"))
    df$tissue = NULL
  
  return(df[,-1, drop=F])
}

# internal: don't call directly, use getData()
getSensitivity = function(ds) {
  sen=NULL
  if(ds$datasource == "gt") {
    if(listItemEquals(ds, 'tissue', 'skin')) {
    
      if(getListItem(ds,'version')=="ic50") sen = sensitivity_melanoma_ic50
      else if(getListItem(ds,'version')=="maxinh") sen = sensitivity_melanoma_maxinh
      else sen = sensitivity_melanoma_viability
    } else if(listItemEquals(ds, 'tissue', 'endometrium')) {
      sen = sensitivity_endometrial
    }
  } else if(ds$datasource =="ccle") {
    sen = sen_ccle
  } else if(ds$datasource =="ctrp") {
    sen = sen_ctrp
  }
  return(sen)
}

#internal: don't call directly, use getData()
getRNAseq = function(ds) {
  rnaseq=NULL
  if(ds$datasource=="gt") {
    if(listItemEquals(ds, 'tissue', 'skin')) {
      rnaseq = rnaseq_melanoma
    } else if(listItemEquals(ds, 'tissue', 'endometrium')) {
      rnaseq = rnaseq_endometrial
    } else { return(NULL) }
    if(getListItem(ds,'version')=="cosmic") {
      sel = (colnames(rnaseq) %in% cosmic_genesym) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    } else if(getListItem(ds,'version')=="rppaskin") {
      sel = (colnames(rnaseq) %in% gt_rppa_skin_genesym) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    } else if(grepl("speed1",getListItem(ds,'version'))) {
      pathway = gsub("speed1", "", getListItem(ds,'version'))
      if(pathway=="all") {
        speed1genes = unique(speed1$gene_symbol)
      } else {
        speed1genes = unique(speed1[speed1$pathway==pathway,"gene_symbol"])
      } 
      sel = (colnames(rnaseq) %in% speed1genes) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    } else if(grepl("random",getListItem(ds,'version'))) {
      num = as.integer(gsub("random", "", getListItem(ds,'version')))
      if(is.na(num)) return(NULL)
      random_genes = sample(setdiff(colnames(rnaseq), c("Cell.Line", "tissue")), num)
      sel = (colnames(rnaseq) %in% random_genes) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    }
  } else if(ds$datasource == "ccle") {
    rnaseq = rnaseq_ccle
    if(getListItem(ds,'version')=="cosmic") {
      sel = (str_split(colnames(rnaseq),"\\.", simplify = T)[,1] %in% cosmic) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    } else if(getListItem(ds,'version')=="rppaccle") {
      sel = (str_split(colnames(rnaseq),"\\.", simplify = T)[,1] %in% rppa_ccle_ensg) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    } else if(grepl("speed1",getListItem(ds,'version'))) {
      pathway = gsub("speed1", "", getListItem(ds,'version'))
      if(pathway=="all") {
        speed1genes = unique(speed1_ensembl$ensembl_gene_id)
      } else {
        speed1genes = unique(speed1_ensembl[speed1_ensembl$pathway==pathway,"ensembl_gene_id"])
      }
      sel = (str_split(colnames(rnaseq),"\\.", simplify = T)[,1] %in% speed1genes) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    } else if(grepl("random",getListItem(ds,'version'))) {
      num = as.integer(gsub("random", "", getListItem(ds,'version')))
      if(is.na(num)) return(NULL)
      random_genes = sample(setdiff(colnames(rnaseq), c("Cell.Line", "tissue")), num)
      sel = (colnames(rnaseq) %in% random_genes) | (colnames(rnaseq) %in% c("Cell.Line", "tissue"))
      rnaseq = rnaseq[ , sel ]
    }
    
  }
  return(rnaseq)
}

#internal: don't call directly, use getData()
getTissue = function(ds) {
  tissues=NULL
  if(ds$datasource=="gt") {
    tissues = data.frame(Cell.Line=rownames(sensitivity_melanoma_viability), tissue="skin")
  } else if(ds$datasource == "ccle") {
    tissues = data.frame(Cell.Line=sen_ccle$Cell.Line, tissue=sen_ccle$tissue)
  } else if(ds$datasource == "ctrp") {
    tissues = data.frame(Cell.Line=sen_ctrp$Cell.Line, tissue=sen_ctrp$tissue)
  }
  rownames(tissues) = tissues$Cell.Line
  tissues = tissues[tissues$tissue!="",]
  return(tissues)
}

# internal: don't call directly, use getData()
getRPPA = function(ds) {
  rppa=NULL
  if(ds$datasource=="gt") {
    if(listItemEquals(ds, 'tissue', 'skin') & !listItemEquals(ds, 'version', 'endosel')) {
      rppa = rppa_gt_skin
      rppa_dmso = rppa_gt_skin[rppa_gt_skin$Tx=="DMSO",]
    } else if(listItemEquals(ds, 'tissue', 'skin') & listItemEquals(ds, 'version', 'endosel')) {
      rppa = rppa_gt_skin_endosel
      rppa_dmso = rppa_gt_skin_endosel[rppa_gt_skin_endosel$Tx=="DMSO",]
    } else if(listItemEquals(ds, 'tissue', 'endometrium')) {
      rppa = rppa_gt_endo
      rppa_dmso = rppa_gt_endo[rppa_gt_endo$Tx=="DMSO",]
    }
    if(getListItem(ds,'Tx')=="1hr") { 
      rppa = rppa[rppa$Tx=="1hr Cobi",] 
    } else if(getListItem(ds,'Tx')=="fc1hr") { rppa = rppa[rppa$Tx=="1hr Cobi",]; rppa[,-c(1:2)]=rppa[,-c(1:2)]/rppa_dmso[,-c(1:2)] 
    } else if(getListItem(ds,'Tx')=="48hr") { rppa = rppa[rppa$Tx=="48hr Cobi",] 
    } else if(getListItem(ds,'Tx')=="fc48hr") { rppa = rppa[rppa$Tx=="48hr Cobi",]; rppa[,-c(1:2)]=rppa[,-c(1:2)]/rppa_dmso[,-c(1:2)] 
    } else { rppa = rppa_dmso }
    rppa$Tx = NULL
    rppa[,-1] = log2(rppa[,-1])
  } else if(ds$datasource=="ccle") {
    rppa = rppa_ccle
  }
  rppa = data.frame(rppa, check.names=F)
  rownames(rppa) = rppa$Cell.Line
  return(rppa)
}

# internal: don't call directly, use getData()
getMutations = function(ds) {
  mutation=NULL
  if(ds$datasource=="gt") {
    if(listItemExists(ds, 'tissue') && !listItemEquals(ds, 'tissue', 'skin'))
      return(NULL)
    
    if(getListItem(ds,'version')=="specPt") {
      mutation = mutation_specPt
    } else if(getListItem(ds,'version')=="gene") {
      mutation = mutation_gene
    } else if(getListItem(ds,'version')=="signature"){
      mutation = mutation_unspecPt
      mutation = mutation[,grepl("Cell.Line|BRAF|NRAS|KRAS", colnames(mutation))]
      BRAF = dplyr::select(mutation, num_range("BRAF_",140753335:140753337))
      BRAF = apply(BRAF,1,max)
      BRAFother = dplyr::select(dplyr::select(mutation,starts_with("BRAF_")), -num_range("BRAF_",140753335:140753337))
      BRAFother = apply(BRAFother,1,max)
      NRASG12 = dplyr::select(mutation, num_range("NRAS_",114716125:114716127))
      if(ncol(NRASG12)==0) {
        NRASG12 = data.frame(NRASG11=rep(0,length(mutation$Cell.Line)), drop=F)
        rownames(NRASG12) = mutation$Cell.Line
      } 
      NRASG12 = apply(NRASG12,1,max)
      NRASG13 = dplyr::select(mutation, num_range("NRAS_",114716122:114716124))
      if(ncol(NRASG13)==0) {
        NRASG13 = data.frame(NRASG13=rep(0,length(mutation$Cell.Line)), drop=F)
        rownames(NRASG13) = mutation$Cell.Line
      } 
      NRASG13 = apply(NRASG13,1,max)
      NRASQ61 = dplyr::select(mutation, num_range("NRAS_",114713907:114713909))
      if(ncol(NRASQ61)==0) {
        NRASQ61 = data.frame(NRASQ61=rep(0,length(mutation$Cell.Line)), drop=F)
        rownames(NRASQ61) = mutation$Cell.Line
      } 
      NRASQ61 = apply(NRASQ61,1,max)
      NRAS = data.frame(NRASG12=NRASG12, NRASG13=NRASG13, NRASQ61=NRASQ61)
      NRAS = apply(NRAS,1,max)
      KRAS = dplyr::select(mutation, starts_with("KRAS_"))
      KRAS = apply(KRAS,1,max)
      mutation = data.frame(Cell.Line=mutation$Cell.Line, BRAF=BRAF, BRAFother=BRAFother, NRAS=NRAS, KRAS=KRAS)
    } else if(getListItem(ds,'version')=="unspecPt") {
      mutation = mutation_unspecPt
    }  else if(getListItem(ds,'version')=="unspecPt_cosmic") {
      mutation = mutation_unspecPt
      sel =  (str_split(colnames(mutation),"_", simplify = T)[,1] %in% cosmic_genesym) | (colnames(mutation) %in% c("Cell.Line", "tissue"))
      mutation = mutation[,sel]
    } else {
      mutation = mutation_unspecPt
    }  
  } else if(ds$datasource=="ccle") {
    mutation = mutation_ccle
  }
  
  return(mutation)
}

# DRUG TARGETS
getDrugTarget = function(ds, remove_dups=NULL, match_CLs=NULL) {
  drug_targets = NULL
  sen = getData(ds, remove_dups=remove_dups, match_CLs=match_CLs)
  if(ds$datasource=="gt" & ds$datatype=="sen" & listItemEquals(ds, 'tissue', 'skin')) {
    drugs_gt <- c('Cobimetinib', 'Irinotecan',  'Dabrafenib', 'DM1', 'Docetaxel', 'Doxorubicin', 'Apo2L', 'Apo2L+FlagM2', 'BRD6929', 'G02594618', 'G02594618+NA', 'AR00482666', 'Taselisib', 'GDC0575', 'GDC0994', 'Gemcitabine', 'GNE330', 'MMAE', 'Oligomycin', 'Paclitaxel', 'PD901', 'PNU159682', 'SG2507', 'SN34994', 'Trametinib', 'TrichostatinA', 'Vemurafenib')
    drugtargets_gt <- c('MAPK', 'Chemo',  'MAPK', 'Chemo', 'Chemo', 'Chemo', 'Other', 'Other', 'HDAC', 'Other', 'Other', 'MAPK', 'PI3K', 'CHK1', 'MAPK', 'Chemo', 'CHK1', 'Chemo', 'Other', 'Chemo', 'MAPK', 'Chemo', 'Chemo', 'Chemo', 'MAPK', 'HDAC', 'MAPK')
    drug_targets = drugtargets_gt[match(colnames(sen), drugs_gt)]
  } else if(ds$datasource=="gt" & ds$datatype=="sen" & listItemEquals(ds, 'tissue', 'endometrium')) {
    drugs_gt <- c('Cobimetinib', 'Irinotecan',  'Dabrafenib', 'DM1', 'Docetaxel', 'Doxorubicin', 'Apo2L', 'Apo2L+FlagM2', 'BRD6929', 'G02594618', 'G02594618+NA', 'AR00482666', 'Taselisib', 'GDC0575', 'GDC0994', 'Gemcitabine', 'GNE330', 'MMAE', 'Oligomycin', 'Paclitaxel', 'PD901', 'PNU159682', 'SG2507', 'SN34994', 'Trametinib', 'TrichostatinA', 'Vemurafenib')
    drugtargets_gt <- c('MAPK', 'Chemo',  'MAPK', 'Chemo', 'Chemo', 'Chemo', 'Other', 'Other', 'HDAC', 'Other', 'Other', 'MAPK', 'PI3K', 'CHK1', 'MAPK', 'Chemo', 'CHK1', 'Chemo', 'Other', 'Chemo', 'MAPK', 'Chemo', 'Chemo', 'Chemo', 'MAPK', 'HDAC', 'MAPK')
    drugs_gt = c(drugs_gt, c("GDC0326","GDC0623","GDC0853","Ipatasertib","Rapamycin"  ))
    drugtargets_gt = c(drugtargets_gt, c("PI3K", "MAPK", "Other", "PI3K", "PI3K"))
    drug_targets = drugtargets_gt[match(colnames(sen), drugs_gt)]
  } else if(ds$datasource=="ccle" & ds$datatype=="sen") {
    drugs_ccle = drugTargets_ccle$Compound
    drugtargets_ccle = drugTargets_ccle$Target
    matched_drugs = match(drugs_ccle, drugTargets_ccle_curated$drug )
    drugtargets_ccle[ !is.na(matched_drugs)] = drugTargets_ccle_curated$category[matched_drugs[!is.na(matched_drugs)] ]
    drugtargets_ccle[ !is.na(match(drugs_ccle, drugTargets_ccle_curated$drug ))] = drugTargets_ccle_curated$category[match(drugs_ccle, drugTargets_ccle_curated$drug )]
    drug_targets = drugtargets_ccle[match(colnames(sen), drugs_ccle)]
  } else if(ds$datasource=="ctrp" & ds$datatype=="sen") {
    drugs_ctrp = drugTargets_ctrp$cpd_name
    drugtargets_ctrp = drugTargets_ctrp$Target
    matched_drugs = match(drugs_ctrp, drugTargets_ctrp_curated$drug )
    drugtargets_ctrp[ !is.na(matched_drugs)] = drugTargets_ctrp_curated$category[matched_drugs[!is.na(matched_drugs)] ]
    drug_targets = drugtargets_ctrp[match(colnames(sen), drugs_ctrp)]
  }
  return(list(drugs=colnames(sen), drug_targets=drug_targets))
}

rm(".Random.seed")
save.image(file="parse_raw_data.RData")
